@@ -1,15 +1,12 @@
 
 /* pngerror.c - stub functions for i/o and memory allocation
  *
- * Last changed in libpng 1.2.37 [June 4, 2009]
- * Copyright (c) 1998-2009 Glenn Randers-Pehrson
+ * Last changed in libpng 1.2.13 November 13, 2006
+ * For conditions of distribution and use, see copyright notice in png.h
+ * Copyright (c) 1998-2006 Glenn Randers-Pehrson
  * (Version 0.96 Copyright (c) 1996, 1997 Andreas Dilger)
  * (Version 0.88 Copyright (c) 1995, 1996 Guy Eric Schalnat, Group 42, Inc.)
  *
- * This code is released under the libpng license.
- * For conditions of distribution and use, see the disclaimer
- * and license in png.h
- *
  * This file provides a location for all error handling.  Users who
  * need special error handling are expected to write replacement functions
  * and use png_set_error_fn() to use those functions.  See the instructions
@@ -18,23 +15,20 @@
 
 #define PNG_INTERNAL
 #include "png.h"
-#if defined(PNG_READ_SUPPORTED) || defined(PNG_WRITE_SUPPORTED)
 
+#if defined(PNG_READ_SUPPORTED) || defined(PNG_WRITE_SUPPORTED)
 static void /* PRIVATE */
 png_default_error PNGARG((png_structp png_ptr,
   png_const_charp error_message));
-#ifndef PNG_NO_WARNINGS
 static void /* PRIVATE */
 png_default_warning PNGARG((png_structp png_ptr,
   png_const_charp warning_message));
-#endif /* PNG_NO_WARNINGS */
 
 /* This function is called whenever there is a fatal error.  This function
  * should not be changed.  If there is a need to handle errors differently,
  * you should supply a replacement error function and use png_set_error_fn()
  * to replace the error function at run-time.
  */
-#ifndef PNG_NO_ERROR_TEXT
 void PNGAPI
 png_error(png_structp png_ptr, png_const_charp error_message)
 {
@@ -47,29 +41,28 @@ png_error(png_structp png_ptr, png_const_charp error_message)
      {
        if (*error_message == '#')
        {
-           /* Strip "#nnnn " from beginning of error message. */
            int offset;
-           for (offset = 1; offset<15; offset++)
-              if (error_message[offset] == ' ')
+           for (offset=1; offset<15; offset++)
+              if (*(error_message+offset) == ' ')
                   break;
            if (png_ptr->flags&PNG_FLAG_STRIP_ERROR_TEXT)
            {
               int i;
-              for (i = 0; i < offset - 1; i++)
-                 msg[i] = error_message[i + 1];
-              msg[i - 1] = '\0';
-              error_message = msg;
+              for (i=0; i<offset-1; i++)
+                 msg[i]=error_message[i+1];
+              msg[i]='\0';
+              error_message=msg;
            }
            else
-              error_message += offset;
+              error_message+=offset;
        }
        else
        {
            if (png_ptr->flags&PNG_FLAG_STRIP_ERROR_TEXT)
            {
-              msg[0] = '0';
-              msg[1] = '\0';
-              error_message = msg;
+              msg[0]='0';
+              msg[1]='\0';
+              error_message=msg;
            }
        }
      }
@@ -82,20 +75,7 @@ png_error(png_structp png_ptr, png_const_charp error_message)
       use the default handler, which will not return. */
    png_default_error(png_ptr, error_message);
 }
-#else
-void PNGAPI
-png_err(png_structp png_ptr)
-{
-   if (png_ptr != NULL && png_ptr->error_fn != NULL)
-      (*(png_ptr->error_fn))(png_ptr, '\0');
 
-   /* If the custom handler doesn't exist, or if it returns,
-      use the default handler, which will not return. */
-   png_default_error(png_ptr, '\0');
-}
-#endif /* PNG_NO_ERROR_TEXT */
-
-#ifndef PNG_NO_WARNINGS
 /* This function is called whenever there is a non-fatal error.  This function
  * should not be changed.  If there is a need to handle warnings differently,
  * you should supply a replacement warning function and use
@@ -114,19 +94,17 @@ png_warning(png_structp png_ptr, png_const_charp warning_message)
      {
        if (*warning_message == '#')
        {
-           for (offset = 1; offset < 15; offset++)
-              if (warning_message[offset] == ' ')
+           for (offset=1; offset<15; offset++)
+              if (*(warning_message+offset) == ' ')
                   break;
        }
      }
+     if (png_ptr != NULL && png_ptr->warning_fn != NULL)
+        (*(png_ptr->warning_fn))(png_ptr, warning_message+offset);
    }
-   if (png_ptr != NULL && png_ptr->warning_fn != NULL)
-      (*(png_ptr->warning_fn))(png_ptr, warning_message + offset);
    else
-      png_default_warning(png_ptr, warning_message + offset);
+      png_default_warning(png_ptr, warning_message+offset);
 }
-#endif /* PNG_NO_WARNINGS */
-
 
 /* These utilities are used internally to build an error message that relates
  * to the current chunk.  The chunk name comes from png_ptr->chunk_name,
@@ -135,14 +113,11 @@ png_warning(png_structp png_ptr, png_const_charp warning_message)
  * if the character is invalid.
  */
 #define isnonalpha(c) ((c) < 65 || (c) > 122 || ((c) > 90 && (c) < 97))
-static PNG_CONST char png_digit[16] = {
+const static PNG_CONST char png_digit[16] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    'A', 'B', 'C', 'D', 'E', 'F'
 };
 
-#define PNG_MAX_ERROR_TEXT 64
-
-#if !defined(PNG_NO_WARNINGS) || !defined(PNG_NO_ERROR_TEXT)
 static void /* PRIVATE */
 png_format_buffer(png_structp png_ptr, png_charp buffer, png_const_charp
    error_message)
@@ -166,21 +141,20 @@ png_format_buffer(png_structp png_ptr, png_charp buffer, png_const_charp
    }
 
    if (error_message == NULL)
-      buffer[iout] = '\0';
+      buffer[iout] = 0;
    else
    {
       buffer[iout++] = ':';
       buffer[iout++] = ' ';
-      png_memcpy(buffer + iout, error_message, PNG_MAX_ERROR_TEXT);
-      buffer[iout + PNG_MAX_ERROR_TEXT - 1] = '\0';
+      png_strncpy(buffer+iout, error_message, 63);
+      buffer[iout+63] = 0;
    }
 }
 
-#ifdef PNG_READ_SUPPORTED
 void PNGAPI
 png_chunk_error(png_structp png_ptr, png_const_charp error_message)
 {
-   char msg[18+PNG_MAX_ERROR_TEXT];
+   char msg[18+64];
    if (png_ptr == NULL)
      png_error(png_ptr, error_message);
    else
@@ -189,14 +163,11 @@ png_chunk_error(png_structp png_ptr, png_const_charp error_message)
      png_error(png_ptr, msg);
    }
 }
-#endif /* PNG_READ_SUPPORTED */
-#endif /* !defined(PNG_NO_WARNINGS) || !defined(PNG_NO_ERROR_TEXT) */
 
-#ifndef PNG_NO_WARNINGS
 void PNGAPI
 png_chunk_warning(png_structp png_ptr, png_const_charp warning_message)
 {
-   char msg[18+PNG_MAX_ERROR_TEXT];
+   char msg[18+64];
    if (png_ptr == NULL)
      png_warning(png_ptr, warning_message);
    else
@@ -205,8 +176,6 @@ png_chunk_warning(png_structp png_ptr, png_const_charp warning_message)
      png_warning(png_ptr, msg);
    }
 }
-#endif /* PNG_NO_WARNINGS */
-
 
 /* This is the default error handling function.  Note that replacements for
  * this function MUST NOT RETURN, or the program will likely crash.  This
@@ -220,35 +189,26 @@ png_default_error(png_structp png_ptr, png_const_charp error_message)
 #ifdef PNG_ERROR_NUMBERS_SUPPORTED
    if (*error_message == '#')
    {
-     /* Strip "#nnnn " from beginning of error message. */
      int offset;
      char error_number[16];
-     for (offset = 0; offset<15; offset++)
+     for (offset=0; offset<15; offset++)
      {
-         error_number[offset] = error_message[offset + 1];
-         if (error_message[offset] == ' ')
+         error_number[offset] = *(error_message+offset+1);
+         if (*(error_message+offset) == ' ')
              break;
      }
-     if ((offset > 1) && (offset < 15))
+     if((offset > 1) && (offset < 15))
      {
-       error_number[offset - 1] = '\0';
-       fprintf(stderr, "libpng error no. %s: %s",
-          error_number, error_message + offset + 1);
-       fprintf(stderr, "\n");
+       error_number[offset-1]='\0';
+       fprintf(stderr, "libpng error no. %s: %s\n", error_number,
+          error_message+offset);
      }
      else
-     {
-       fprintf(stderr, "libpng error: %s, offset=%d",
-          error_message, offset);
-       fprintf(stderr, "\n");
-     }
+       fprintf(stderr, "libpng error: %s, offset=%d\n", error_message,offset);
    }
    else
 #endif
-   {
-      fprintf(stderr, "libpng error: %s", error_message);
-      fprintf(stderr, "\n");
-   }
+   fprintf(stderr, "libpng error: %s\n", error_message);
 #endif
 
 #ifdef PNG_SETJMP_SUPPORTED
@@ -257,7 +217,7 @@ png_default_error(png_structp png_ptr, png_const_charp error_message)
 #  ifdef USE_FAR_KEYWORD
    {
       jmp_buf jmpbuf;
-      png_memcpy(jmpbuf, png_ptr->jmpbuf, png_sizeof(jmp_buf));
+      png_memcpy(jmpbuf,png_ptr->jmpbuf,png_sizeof(jmp_buf));
       longjmp(jmpbuf, 1);
    }
 #  else
@@ -268,11 +228,12 @@ png_default_error(png_structp png_ptr, png_const_charp error_message)
    PNG_ABORT();
 #endif
 #ifdef PNG_NO_CONSOLE_IO
-   error_message = error_message; /* Make compiler happy */
+   /* make compiler happy */ ;
+   if (&error_message != NULL)
+      return;
 #endif
 }
 
-#ifndef PNG_NO_WARNINGS
 /* This function is called when there is a warning, but the library thinks
  * it can continue anyway.  Replacement functions don't have to do anything
  * here if you don't want them to.  In the default configuration, png_ptr is
@@ -287,38 +248,33 @@ png_default_warning(png_structp png_ptr, png_const_charp warning_message)
    {
      int offset;
      char warning_number[16];
-     for (offset = 0; offset < 15; offset++)
+     for (offset=0; offset<15; offset++)
      {
-        warning_number[offset] = warning_message[offset + 1];
-        if (warning_message[offset] == ' ')
+        warning_number[offset]=*(warning_message+offset+1);
+        if (*(warning_message+offset) == ' ')
             break;
      }
-     if ((offset > 1) && (offset < 15))
+     if((offset > 1) && (offset < 15))
      {
-       warning_number[offset + 1] = '\0';
-       fprintf(stderr, "libpng warning no. %s: %s",
-          warning_number, warning_message + offset);
-       fprintf(stderr, "\n");
+       warning_number[offset-1]='\0';
+       fprintf(stderr, "libpng warning no. %s: %s\n", warning_number,
+          warning_message+offset);
      }
      else
-     {
-       fprintf(stderr, "libpng warning: %s",
-          warning_message);
-       fprintf(stderr, "\n");
-     }
+       fprintf(stderr, "libpng warning: %s\n", warning_message);
    }
    else
 #  endif
-   {
-     fprintf(stderr, "libpng warning: %s", warning_message);
-       fprintf(stderr, "\n");
-   }
+     fprintf(stderr, "libpng warning: %s\n", warning_message);
 #else
-   warning_message = warning_message; /* Make compiler happy */
+   /* make compiler happy */ ;
+   if (warning_message)
+     return;
 #endif
-   png_ptr = png_ptr; /* Make compiler happy */
+   /* make compiler happy */ ;
+   if (png_ptr)
+      return;
 }
-#endif /* PNG_NO_WARNINGS */
 
 /* This function is called when the application wants to use another method
  * of handling errors and warnings.  Note that the error function MUST NOT
@@ -354,7 +310,7 @@ png_get_error_ptr(png_structp png_ptr)
 void PNGAPI
 png_set_strip_error_numbers(png_structp png_ptr, png_uint_32 strip_mode)
 {
-   if (png_ptr != NULL)
+   if(png_ptr != NULL)
    {
      png_ptr->flags &=
        ((~(PNG_FLAG_STRIP_ERROR_NUMBERS|PNG_FLAG_STRIP_ERROR_TEXT))&strip_mode);
